#include "pnmParser.h"

void malloc_pnm(Map m, FILE* pnm){
    int i;
    m->map = pnm_allocarray(m->cols, m->rows);
    for (i=0; i < m->rows; i++){
        pnm_readpnmrow(pnm, &(m->map[i][0]), m->cols, m->maxval, m->format );
    }
}

/* Retorna uma matriz de "colors" com "rows" linhas e "cols" colunas,
 * contendo uma borda nos quatro lados de espessura "border_size"
 */
colors ** get_empty_slice(Map m, int rows, int cols, int border_size){
   int rows_border = rows + 2*border_size;
   int cols_border = cols + 2*border_size;
   int i,j;
   colors ** slice = malloc((rows_border)*sizeof( colors * ));
   for (i = 0; i < rows_border; i++){
      slice[i] = malloc((cols_border)*sizeof( colors ));
   }
   // Limpa o slice
   for ( i = 0; i < rows_border; i++)
      for ( j = 0;  j < cols_border; j++){
         slice[i][j].r = 0;
         slice[i][j].g = 0;
         slice[i][j].b = 0;
      }
   
   return slice;
}

/* Retorna uma parte da imagem, de (start_i, start_j) a (end_i, end_j),
 * inclusive. Também suporta bordas. A origem é no canto superior esquerdo,
 * sendo i a ordenada e, j, a abscissa.
 */
colors ** get_slice(Map m , int start_i, int start_j, int end_i, int end_j, int border_size ){
   int i,j;
   colors ** slice = get_empty_slice(m,end_i - start_i + 1, end_j - start_j + 1 , border_size);
      
   for ( i = 0; i <= (end_i - start_i); i++){
      for ( j = 0;  j <= (end_j - start_j); j++){
         slice[i+border_size][j+border_size].r = PPM_GETR(m->map[i+start_i][j+start_j]);
         slice[i+border_size][j+border_size].g = PPM_GETG(m->map[i+start_i][j+start_j]);
         slice[i+border_size][j+border_size].b = PPM_GETB(m->map[i+start_i][j+start_j]);         
      }
   }
   return slice;
}

/* Copia uma parte da imagem para o mapa, que contem a imagem toda */
void set_slice(Map m, int start_i, int start_j, int end_i, int end_j, int border_size, colors ** slice){
   int i,j;
   xel value;
   for ( i = 0; i <= (end_i - start_i); i++){
      for ( j = 0;  j <= (end_j - start_j); j++){
         value =  slice[i+border_size][j+border_size]; 
         PPM_PUTR(m->map[start_i + i][start_j + j] , value.r);
         PPM_PUTG(m->map[start_i + i][start_j + j] , value.g);
         PPM_PUTB(m->map[start_i + i][start_j + j] , value.b);
      }
   }   
}

/* Exibe informações sobre o arquivo pnm */
void info_pnm(Map m){
    printf("col = %d\n", m->cols);
    printf("row = %d\n", m->rows);
    printf("PNM_FORMAT = %d\n", m->format);
    printf("RPNM_FORMAT  = %d\n", m->format);
}

/* Cria um novo mapa
 * name = nome do arquivo pnm
 * m = mapa
 */
Map new_map(char * name){
    Map m = malloc(sizeof(*m));
    FILE *pnm = pm_openr( name );
    pnm_readpnminit(pnm, &(m->cols), &(m->rows), &(m->maxval), &(m->format));
    malloc_pnm(m, pnm);
    return m;
}

/* Finaliza o uso de pnms */
void close_pnm(Map m, FILE * pnm){
    pm_close(pnm);
    pnm_freearray(m->map, m->rows);
}

/* Salva a imagem */
void save_pnm(Map m, char * name){
    int i;   
    FILE* file = pm_openw (name);
    pnm_writepnminit(file, m->cols, m->rows, m->maxval, m->format, 0);
    for (i=0; i < m->rows; i++){
        pnm_writepnmrow( file, &(m->map[i][0]), m->cols, m->maxval, m->format, 0);
    }
    pm_close( file );
}
